/*
 * \file   main.c
 *
 * \brief  Main routine for the bootloader for FR5739
 *
 */
/* --COPYRIGHT--,BSD
 * Copyright (c) 2012, Texas Instruments Incorporated
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * *  Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 *
 * *  Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 *
 * *  Neither the name of Texas Instruments Incorporated nor the names of
 *    its contributors may be used to endorse or promote products derived
 *    from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO,
 * THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
 * PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR
 * CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,
 * EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
 * PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS;
 * OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY,
 * WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR
 * OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE,
 * EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 * --/COPYRIGHT--*/
//
//  Include files
//
#include "msp430.h"
#include "TI_MSPBoot_Common.h"
#include "TI_MSPBoot_CI.h"
#include "TI_MSPBoot_MI.h"
#include "TI_MSPBoot_AppMgr.h"

////////////////////////////////////////////////////////////////////////////////////
///////////////////Deep TEST////////////////////
#include <stdint.h>


#define App1Array_SIZE   7168

//const uint16_t App1Array_Addr[1] = {
//    0xde00,	// Address segment0
//};
//
//const uint16_t App1Array_Size[1] = {
//7168,   // Size segment0
//};

uint8_t App1Array_0[] = {
		0x86,0xc4,0xff,0xff,0x2a,0x14,0x40,0x18,0x1a,0x42,0x5c,0x01,0x8f,0x00,0x00,0x00,
		0x9f,0x00,0x00,0x00,0x13,0x24,0x89,0x00,0x00,0x00,0x88,0x00,0x00,0x00,0x0c,0x3c,
		0x0c,0x09,0x7f,0x4c,0x5f,0x06,0x00,0x18,0x5f,0x4f,0x00,0x00,0xa9,0x00,0x04,0x00,
		0x0d,0x09,0x4f,0x13,0xa9,0x00,0x04,0x00,0xd9,0x08,0xf2,0x23,0x7a,0xc2,0x3a,0xd0,
		0x08,0x5a,0x40,0x18,0x82,0x4a,0x5c,0x01,0x8f,0x00,0x00,0x00,0x9f,0x00,0x00,0x00,
		0x09,0x24,0x8a,0x00,0x00,0x00,0x03,0x3c,0x6a,0x13,0xaa,0x00,0x04,0x00,0x9a,0x00,
		0x00,0x00,0xfa,0x23,0x28,0x16,0x10,0x01,0xb2,0x40,0x80,0x5a,0x5c,0x01,0xd2,0xc3,
		0x02,0x02,0xd2,0xd3,0x04,0x02,0x92,0xc3,0x30,0x01,0xd2,0xe3,0x02,0x02,0x0d,0x14,
		0x3d,0x40,0x32,0x82,0x1d,0x83,0xfe,0x23,0x0d,0x16,0xf7,0x3f,0x03,0x43,0x81,0x00,
		0x00,0x20,0xb0,0x13,0xae,0xde,0x0c,0x93,0x02,0x24,0xb0,0x13,0x04,0xde,0x0c,0x43,
		0xb0,0x13,0x68,0xde,0xb0,0x13,0xb2,0xde,0x32,0xd0,0x10,0x00,0xfd,0x3f,0x1c,0x43,
		0x10,0x01,0x03,0x43,0xff,0x3f
};
uint8_t App1Array_1[] = {
		0xa8,0xde,0xa8,0xde,
		0xa8,0xde,0xa8,0xde,0xa8,0xde,0xa8,0xde,0xa8,0xde,0xa8,0xde,0xa8,0xde,0xa8,0xde,
		0xa8,0xde,0xa8,0xde,0xa8,0xde,0xa8,0xde,0xa8,0xde,0xa8,0xde,0xa8,0xde,0xa8,0xde,
		0xa8,0xde,0xa8,0xde,0xa8,0xde,0xa8,0xde,0xa8,0xde,0xa8,0xde,0xa8,0xde,0x8e,0xde
};
//const uint8_t *App1Array_Ptr[1] = {
//App1Array_0,
//};
uint8_t i=0;
uint16_t addr1 = 0xde00;
///////////////////Deep TEST END////////////////
////////////////////////////////////////////////////////////////////////////////////
//
//  Local function prototypes
//
static void clock_init(void);
static void HW_init(void);
static void MPU_init(void);
static uint8_t TI_MSPBoot_MI_WriteByteDirect_Main(uint16_t addr, uint8_t data);
static void TI_MSPBoot_MI_EraseAppDirect_Main(uint8_t DownArea);
/******************************************************************************
 *
 * @brief   Main function
 *  - Initializes the MCU
 *  - Selects whether to run application or bootloader
 *  - If bootloader:
 *      - Initializes the peripheral interface
 *      - Waits for a command
 *      - Sends the corresponding response
 *  - If application:
 *      - Jump to application
 *
 *  @note   USI interrupts are enabled after this function
 * @return  none
 *****************************************************************************/
int main_boot( void )
{
    // Stop watchdog timer to prevent time out reset
    WDTCTL = WDTPW + WDTHOLD;

    // Configure GPIO
	P1OUT &= ~0x01;                 // Clear P1.0 output latch for a defined power-on state
	P1DIR |= 0x01;                  // Set P1.0 to output direction

	P4OUT &= ~0x80;                 // Clear P1.0 output latch for a defined power-on state
	P4DIR |= 0x80;                  // Set P1.0 to output direction

	PM5CTL0 &= ~LOCKLPM5;           // Disable the GPIO power-on default high-impedance mode

    // Initialize MPU
    MPU_init();

    // Initialize MCU
    HW_init();
    clock_init();

	P1OUT ^= BIT0;                // Toggle LED
	__delay_cycles(500000);
	P1OUT ^= BIT0;
	__delay_cycles(500000);

	//TI_MSPBoot_MI_EraseAppDirect_Main(0);

	addr1 = 0xde00;
	for(i=0;i<=182;i++)
	{
		TI_MSPBoot_MI_WriteByteDirect_Main(addr1++, App1Array_0[i]);
	}

	addr1 = 0xf9cc;
	for(i=0;i<52;i++)
	{
		TI_MSPBoot_MI_WriteByteDirect_Main(addr1++, App1Array_1[i]);
	}


    P4OUT ^= BIT7;
	__delay_cycles(500000);
	// Validate the application and jump if needed
    //if (TI_MSPBoot_AppMgr_ValidateApp() == TRUE_t)
        TI_MSPBoot_APPMGR_JUMPTOAPP();

    P4OUT ^= BIT7;

    TI_MSPBoot_CI_Init();      // Initialize the Communication Interface

#ifndef NDEBUG
    PJDIR |= BIT0|BIT1;     // Used for debugging purposes to show entry to MSPBoot
    PJOUT |= BIT0|BIT1;
#endif

    while(1)
    {

//        MPUCTL0 = MPUPW | MPUENA;   // Enable access to MPU registers
//        MPUSAM |= MPUSEG3WE;        // Enable Write access
//
//        *(uint8_t*)(0xde00)= App1Array_0[0];
//
//        MPUSAM &= ~MPUSEG3WE;       // Disable Write access
//        MPUCTL0_H = 0x00;           // Disable access to MPU registers

        // Poll PHY and Data Link interface for new packets
        TI_MSPBoot_CI_PHYDL_Poll();

        // If a new packet is detected, process it
        if (TI_MSPBoot_CI_Process() == RET_JUMP_TO_APP)
        {
            // If Packet indicates a jump to App
            TI_MSPBoot_AppMgr_JumpToApp();
        }
#ifdef NDEBUG
        // Feed the dog every ~1000ms
        WATCHDOG_FEED();
#endif
    }


}

/******************************************************************************
 *
 * @brief   Initializes the MSP430 Clock
 *
 * @return  none
 *****************************************************************************/
//inline static void clock_init(void)
static void clock_init(void)
{
    CSCTL0_H = 0xA5;
    CSCTL1 = DCOFSEL0 + DCOFSEL1;                         // Set DCO = 8Mhz
    CSCTL2 = SELA__VLOCLK + SELM__DCOCLK + SELS__DCOCLK;  // set ACLK = VLO
                                                          // MCLK=SMCLK=DCO
    
#if (MCLK==1000000)
    CSCTL3 = DIVA__1 + DIVS__8 + DIVM__8;                 // Divide DCO/8
#elif (MCLK==4000000)
    CSCTL3 = DIVA__1 + DIVS__2 + DIVM__2;                 // Divide DCO/2
    #elif (MCLK==8000000)
    CSCTL3 = DIVA__1 + DIVS__1 + DIVM__1;                 // Divide DCO/1
#else
#error "Please define a valid MCLK or add configuration"
#endif

}


/******************************************************************************
 *
 * @brief   Initializes the basic MCU HW
 *
 * @return  none
 *****************************************************************************/
static void HW_init(void)
{
    // Just initialize S2 button to force BSL mode
    P4OUT |= BIT1;
    P4REN |= BIT1;
}

/******************************************************************************
 *
 * @brief   Initializes the Memory Protection Unit of FR5739
 *          This allows for HW protection of Bootloader area
 *
 * @return  none
 *****************************************************************************/
static void MPU_init(void)
{
    // These calculations work for FR5739 (check user guide for MPUSEG values)
    #define MPUB1 (((APP_START_ADDR)&0x3F00)>>9)
    #define MPUB2 (((APP_END_ADDR+1)&0x3F00)>>9)
    
    // Enable access to MPU registers
    MPUCTL0 = MPUPW;
    // Seg1 = 0x0000 - App , Seg2 = App - Boot, Seg3 = Boot - 0xFFFF
    //MPUSEG = (MPUB2 << 8) | MPUB1;
    MPUSEGB1 = MPUB1;
    MPUSEGB2 = MPUB2;
    // Seg 3 doesn't have write-access and generates a PUC
    MPUSAM = 0x7D77;
    // Enable MPU
    MPUCTL0 = MPUPW |MPUENA;
    // Disable access to MPU Registers
    MPUCTL0_H = 0x00;
    
}
/******************************************************************************
 *
 * @brief   Write a Byte Directly to Flash memory
 *          The bootloader is protected using MPU but all interrupts (except for
 *          Vector) can be reprogrammed
 *
 * @param  addr     Address of the Byte being written in Flash
 * @param  data     Byte being written
 *
 * @return  RET_OK when sucessful,
 *          RET_PARAM_ERROR if address is outside of Application area
 *****************************************************************************/
uint8_t TI_MSPBoot_MI_WriteByteDirect_Main(uint16_t addr, uint8_t data)
{
    //*(uint8_t *) addr = data;
    //if ((addr >= APP_VECTOR_TABLE) && (addr < APP_RESET_VECTOR_ADDR))
    {
        // If address is an interrupt vector, copy directly to interrupt table
    //    addr = (addr - APP_VECTOR_TABLE) + BOOT_VECTOR_TABLE;
        MPUCTL0 = MPUPW | MPUENA;   // Enable access to MPU registers
        MPUSAM |= MPUSEG3WE;        // Enable Write access
        *(uint8_t *) addr = data;   // Write the vector
        MPUSAM &= ~MPUSEG3WE;       // Disable Write access
        MPUCTL0_H = 0x00;           // Disable access to MPU registers
    }
    return RET_OK;
}


void TI_MSPBoot_MI_EraseAppDirect_Main(uint8_t DownArea)
{
    uint16_t addr;
    uint16_t Start, End;
    extern uint16_t _FLASHDOWN_START; // Define Address of Download area
    extern uint16_t _App_EndDown; // Define Address of Download area

    if (DownArea==0)
    {
        Start = APP_START_ADDR;
        End = APP_END_ADDR;
    }
    else
    {
        Start = ((uint16_t )&_FLASHDOWN_START);
        End = ((uint16_t )&_App_EndDown);
    }

    for (addr = End; addr >=Start; addr--)
    {
        MPUCTL0 = MPUPW | MPUENA;   // Enable access to MPU registers
        MPUSAM |= MPUSEG3WE;        // Enable Write access
        *(uint8_t *) (addr) = 0xFF;
        MPUSAM &= ~MPUSEG3WE;       // Disable Write access
        MPUCTL0_H = 0x00;           // Disable access to MPU registers
    }

}


